@@ -1,4 +1,5 @@
 import struct
+import ctypes
 from typing import Optional
 from enum import Enum
 
@@ -15,8 +16,17 @@ class Architecture(object):
     def write(self, addr: int, data: bytes):
         raise NotImplementedError()
 
-    def read_ptr(self, addr):
-        return struct.unpack("<Q" if self._x64 else "<I", self.read(addr, self.ptr_size()))[0]
+    def read_char(self, addr):
+        return struct.unpack("<b", self.read(addr, 1))[0]
+
+    def read_short(self, addr):
+        return struct.unpack("<h", self.read(addr, 2))[0]
+
+    def read_long(self, addr):
+        return struct.unpack("<i", self.read(addr, 4))[0]
+
+    def read_byte(self, addr):
+        return struct.unpack("<B", self.read(addr, 1))[0]
 
     def read_ushort(self, addr):
         return struct.unpack("<H", self.read(addr, 2))[0]
@@ -24,18 +34,27 @@ class Architecture(object):
     def read_ulong(self, addr):
         return struct.unpack("<I", self.read(addr, 4))[0]
 
-    def read_long(self, addr):
-        return struct.unpack("<i", self.read(addr, 4))[0]
+    def read_ptr(self, addr):
+        return struct.unpack("<Q" if self._x64 else "<I", self.read(addr, self.ptr_size()))[0]
 
-    def read_short(self, addr):
-        return struct.unpack("<h", self.read(addr, 2))[0]
+    def write_char(self, addr, value):
+        self.write(addr, struct.pack("<b", value))
 
-    def write_ulong(self, addr, value):
-        self.write(addr, struct.pack("<I", value))
+    def write_short(self, addr, value):
+        self.write(addr, struct.pack("<h", value))
 
     def write_long(self, addr, value):
         self.write(addr, struct.pack("<i", value))
 
+    def write_byte(self, addr, value):
+        self.write(addr, struct.pack("<B", value))
+
+    def write_ushort(self, addr, value):
+        self.write(addr, struct.pack("<H", value))
+
+    def write_ulong(self, addr, value):
+        self.write(addr, struct.pack("<I", value))
+
     def write_ptr(self, addr, value):
         self.write(addr, struct.pack("<Q" if self._x64 else "<I", value))
 
@@ -54,6 +73,12 @@ class Architecture(object):
 
         return data.decode(encoding)
 
+    def ptr_type(self, t=None):  # TODO: implement type
+        return ctypes.c_uint64 if self._x64 else ctypes.c_uint32
+
+    def alignment(self):
+        return 16 if self._x64 else 8
+
 class PVOID:
     def __init__(self, ptr: int, arch: Architecture):
         self.ptr = ptr
@@ -88,7 +113,10 @@ class PVOID:
         return self.ptr != other
 
     def __str__(self):
-        return f"0x{self:X}"
+        return hex(self.ptr)
+
+    def read_byte_str(self, size):
+        return bytes(self.read(size))
 
     def read_str(self, size, encoding="utf8"):
         return self.read(size).decode(encoding)
@@ -104,6 +132,12 @@ class PVOID:
     def write_ptr(self, value: int):
         return self.arch.write_ptr(self.ptr, value)
 
+    def write_ulong(self, value: int):
+        return self.arch.write_ulong(self.ptr, value)
+
+    def read_ulong(self):
+        return self.arch.read_ulong(self.ptr)
+
     def deref(self):
         assert self.type is not None
         return self[0]
